require("ggplot2")

numberFormat <- function(num, digits=0){
	return(prettyNum(round(num,digits),big.mark=",",scientific=FALSE))
}

percentFormat <- function(decimal, digits=1, escapePercent=TRUE){
	
	if(escapePercent)
		percent = "\\%"
	else
		percent = "%"
	
	return(paste0(formatC(100*decimal,format="f",digits=digits),percent))
}

pValFormat <- function(val,eps=.001){
	return(gsub(" 0[.]",".",format.pval(val,eps=eps)))
}

ciFormat <- function(interval){
	return(paste0("[",percentFormat(interval$lower,digits=2,escapePercent=FALSE),",",
					percentFormat(interval$upper,digits=2,escapePercent=FALSE),"]"))
}

chiSqFormat <- function(test,eps=.001){
	
	ps = ifelse(test$p.value<.05,
			paste0('p',pValFormat(test$p.value,eps)),
			'n.s.'
	)
	
	return(paste0('($\\chi^2(df=',test$parameter,
					', n=',numberFormat(sum(test$observed),0),
					') = ',numberFormat(test$statistic,0),
					', ',ps,'$)'))	
}

newcommand <- function(name, contents ){
	cat(paste0("\\newcommand{\\",name,"}{",contents,"\\xspace}","\n"))
}

newcommandNum <- function(name, dataframe ){
	newcommand(name,numberFormat(dataframe[1,1]))
}

newcommandProportion <- function(name, numerator, denominator, digits=1){
	newcommandPercent(name,numerator/denominator,digits)
}

newcommandPercent <- function(name, decimal, digits=1){
	newcommand(name,percentFormat(decimal,digits))
}

notes <- function(note){
	cat(paste0("% ",note,"\n"))
}

closeAllConnections <- function(){
	cons <- dbListConnections(MySQL())
	for(con in cons)
		dbDisconnect(con)
}

# from http://stackoverflow.com/questions/11418920/how-to-send-a-query-from-a-sql-file
sqlImport <- function(file) {
	sqlStr <- scan(file, sep = "\n", what = "character", quiet=TRUE)
	sqlStr <- gsub("^[ \t]+","", sqlStr)
	sqlStr <- gsub("^--.*","", sqlStr)
	sqlStr <- paste(sqlStr, collapse=" ")
	return(sqlStr)
}

maxY <- function(){0.95}
minY <- function(){0.6}

y_ar_scale <- function(){
	return(scale_y_continuous(expand = c(0,0), limit=c(minY(), maxY()), labels = scales::percent))
}