library(DBI)
library(RMySQL)
library(xtable)
library(binom)
library(reshape)

sourceFrom <- function(file){
	source(paste0("Publication - Conference - Gender Bias in Pull Requests/r/",file))
}

sourceFrom("common.R")

createConnection <- function(){	
m<-dbDriver("MySQL");
con<-dbConnect(m,user=Sys.getenv('MYSQL_USER'),
	password=Sys.getenv('MYSQL_PASSWORD'),
	host=Sys.getenv('MYSQL_SERVER'),
	port=Sys.getenv('MYSQL_SERVER_PORT'),
	dbname='ghtorrent', client.flag = CLIENT_MULTI_STATEMENTS);
return(con)
}

con <- createConnection()

# 83 seconds
notes("Earliest and latest pull request date?")
dataframe<-dbGetQuery(con, 
	"SELECT DATE_FORMAT(min(created_at),'%M %e, %Y'),
		DATE_FORMAT(max(created_at),'%M %e, %Y') 
	FROM pull_request_history")
newcommand("earliestPullRequest",dataframe[1,1])
newcommand("latestPullRequest",dataframe[1,2])

#15 seconds
notes("GHTorrent users with email addresses")
emails<-dbGetQuery(con, 
	"SELECT COUNT(*) FROM users WHERE 
	email LIKE '%_@__%.__%'")
newcommandNum("usersWithEmails",emails)

# fast
notes("Male and female count")
genderCount<-dbGetQuery(con, 
		"select count(*) from g_gplus where 
			gender in ('male','female')")
newcommandNum("maleAndFemaleCount",genderCount)

# fast
notes("Percent of email users with genders")
newcommandProportion("percentWithGenders",genderCount[1,1],emails[1,1])

## 10 seconds
sourceFrom("identicons.R");
identicons(con)

# 8 seconds
notes("What percent of pull requests have an abnormal state?")
dataframe<-dbGetQuery(con, "SELECT state,count(*) FROM ghtorrent.g_pull_closes2 
							WHERE state NOT IN ('issue')
							GROUP BY state")
pullStateAbnormal <- subset(dataframe,!is.element(state,c('open','closed','merged')),select="count(*)")
newcommandProportion("pullsAbnormalPercent",sum(pullStateAbnormal),sum(dataframe$'count(*)'),2)

# 15 minutes or less
source("perceived_gender_36/second.R")
result <- profileAnalysis(con)

# 6 minutes
sourceFrom("overallAnalysis.R")
overallAnalysis(con)

## up to 30 minutes, depending on network speed 
sourceFrom("gPlusVsNoGPlus.R")
gPlusAnalysis(con)

# 1 minute
source("g_results/graph_acceptance_rate_vs_pull_request_count.R")
rateOverTime(con)

# 4 minutes
source("acceptance_distribution_56/second.R")
data <- acceptanceDistribution(con)

# 10 minute
source("code_size_22/analysis.R")
codeSizeByGender(con)

# 3 minutes 
sourceFrom("projects.R")
d <- projectAnalysis(con)

#30 minutes
source("g_results/references.R")
refAnalysis(con)
		
data<-dbGetQuery(con, "SELECT files_changed,AVG(s) mean FROM
				(	SELECT IF(state='merged',1,0) s,files_changed 
				FROM g_pull_closes2
				WHERE 	files_changed in (10,20) AND
				state in ('closed','merged','open')
				) as w        
				GROUP BY files_changed;")

newcommand("acceptanceDropIncreaseFiles",percentFormat(data[1,2]-data[2,2]))	

data<-dbGetQuery(con, "select avg(sm) from
				(select if(self_merge,1,0) sm from g_pull_closes2 
				where creator_type not in ('Owner','Collaborator') and state='merged') as w")

newcommand("outsiderSelfMergePercent",percentFormat(data[1,1]))	

newcommand("numPanelAnalyzed",dbGetQuery(con, "select count(*) from gender_panel_results")[1,1])
newcommand("numPanelIdd",dbGetQuery(con, "select count(*) from gender_panel_results where identifiable=1"))


#3 minutes
source("pr_file_types_issue_50/third.R")
fileAnalysis(con)

#Hours to complete!
source("psm_125/psm_results.R")
psm.overall(con=con)
dbDisconnect(con)

#TODO could be faster by re-using the underlyign data source. 
#		but reading from database 2x 180MB isn't too slow.
con <- createConnection()
psm.identifiable(con=con)
dbDisconnect(con)