#This (and pulls_for_everyone_issue_137/create_merge_report.sql) gotta be run first
#CREATE TABLE sandbox.openers(PRIMARY KEY (pull_request_id)) AS
#(SELECT pull_request_id,actor_id FROM ghtorrent.pull_request_history where action='opened'
#GROUP BY pull_request_id
#having MIN(created_at));
#
#ALTER TABLE sandbox.openers add key (actor_id);

library(plyr)

genderTransformer <- function (gender){
	ifelse(is.na(gender),
			'User not on Google+',
			ifelse(gender=='Male' | gender=='Female',
					paste0("User identifies as `",gender,"' on Google+"),
					ifelse(gender=='N/A',
							' User has no gender listed on Google+',
							ifelse(gender=='Decline to State',
									"User lists `Declined to State' for gender on Google+",
									'User lists other gender on Google+'))))
}

gPlusAnalysis <- function(con){

	# (select * from sandbox.merge_report limit 1000) mr
	pulls <- dbGetQuery(con,"SELECT 
					actor_id,
					gender,
					IF(isMerged, 1, 0) mgd,
					IF(isMerged, 0, 1) umgd,
					base_repo_id
					FROM
					sandbox.merge_report mr
					LEFT JOIN sandbox.openers op ON mr.pull_request_id=op.pull_request_id
					LEFT JOIN g_gplus gp ON op.actor_id = gp.userId
					LEFT JOIN pull_requests pr on pr.id = mr.pull_request_id")
	
	pulls <- transform(pulls,genderCat=genderTransformer(gender))
	
	descriptives <- ddply(pulls,c("genderCat"),summarize,
			accept=sum(mgd)/(sum(mgd)+sum(umgd)),
			conf=ciFormat(binom.confint(sum(mgd),(sum(mgd)+sum(umgd)),0.95,"exact")),
			users=length(unique(actor_id)),
			pulls=sum(mgd)+sum(umgd))
	
	# create per-user summaries
	reposPerUser <- ddply(pulls,c("genderCat","actor_id"),summarize,repo_count=length(unique(base_repo_id)))
	repo_quantiles <- ddply(reposPerUser,c("genderCat"),function(x) quantile(x$repo_count,c(.5,.75,.9),na.rm=TRUE))
	
	result <- merge(x=descriptives,y=repo_quantiles,by="genderCat", all=TRUE)
	result <- result[order(result$users,decreasing=TRUE),]
	result <- transform(result, check.names=FALSE, 
			`Gender Category`= genderCat,
			Users = numberFormat(users),
			`Pull Requests` = numberFormat(pulls),
			`Acceptance Rate` = percentFormat(accept,escapePercent=FALSE),
			`95% Confidence Interval` = conf)
		
	makeTableCommand <- function(name,table){
		newcommand(name,print(table,
						table.placement="t!",
						hline.after=c(-1:nrow(table)),
						include.rownames=FALSE,
						print.results=FALSE,
						comment=FALSE))		
	}
	
	gplusAcceptTable <- xtable(result[c("Gender Category","Users","Pull Requests","Acceptance Rate","95% Confidence Interval")],
			label="tbl:gplusAcceptance",
			caption="Acceptance rates for GitHub users not linked to Google+ (top row) versus 
					those who are linked (bottom rows), by stated gender. Right
					three columns indicate the percentiles of the number of 
					projects contributed to.")
	align(gplusAcceptTable) <- "rR{35mm}|rrrr"
	makeTableCommand("gplusAcceptTable",gplusAcceptTable)
	
	gplusProjTable <- xtable(result[c("Gender Category","Users","Pull Requests","50%","75%","90%")],
			label="tbl:gPlusProj",
			caption="Percentiles of the number of 
					projects contributed to for 
					GitHub users not linked to Google+ (top row) versus 
					those who are linked (bottom rows), by stated gender.")
	align(gplusProjTable) <- "rR{35mm}|rrrrr"
	makeTableCommand("gplusProjPercentiles",gplusProjTable)
	
	countUsers <- function(gender){
		result[which(result$genderCat==genderTransformer(gender)),]$users		
	}
	
	newcommand('otherGenderCount',numberFormat(countUsers('Other gender listed')))
	newcommand('totalMenAndWomen',numberFormat(countUsers('Other gender listed') + countUsers('Male') + countUsers('Female')))
		
	return(result)
}