commonAnalysis <- function(con, excludeInsiders=FALSE, restrictLicense=FALSE){

	mainResult<-dbGetQuery(con, 
		paste0("SELECT if(gender='Male','Men','Women') as `Gender`,
			sum(if(state='open',1,0)) as Open,
			sum(if(state='closed',1,0)) as Closed,
	        sum(if(state='merged',1,0)) as Merged
		FROM ghtorrent.g_pull_closes2 gpc
			",ifelse(restrictLicense,"left join g_project_licenses gpl on gpc.repo_id=gpl.repo_id 
            left join g_licenses gl on gl.license=gpl.license","")," 
			WHERE gender IN('Female','Male')
			",ifelse(excludeInsiders," AND creator_type NOT IN ('Owner','Collaborator') ",""),
			  ifelse(restrictLicense," AND `License Type`='Open Source' ",""),
		"GROUP BY gender;"))
	
	gender1row <- c(mainResult[1,]$Merged,mainResult[1,]$Open+mainResult[1,]$Closed)
	gender2row <- c(mainResult[2,]$Merged,mainResult[2,]$Open+mainResult[2,]$Closed)
	
	mainResult <- transform(mainResult, check.names=FALSE,
			`Merge Rate`= percentFormat(Merged / (Open+Closed+Merged),escapePercent=FALSE),
			`95% Confidence Interval` = ciFormat(binom.confint(Merged,Open+Closed+Merged,0.95,"exact")),
			Open = numberFormat(Open),
			Closed = numberFormat(Closed),
			Merged = numberFormat(Merged))
	
	resultTable <- xtable(mainResult);
	
	align(resultTable) <- "rr|rrr|rr"
	
	bold <- function(x){
		oldSanitize <- sanitize(x,type="latex")
		# Everythign in the merge rate column
		return(ifelse(grepl("(Rate|%$)",oldSanitize),
						paste0('{\\textbf{ ',oldSanitize, '}}'),oldSanitize))
	}
	
	outsider = ifelse(excludeInsiders,"Outsider","")
	restrict = ifelse(restrictLicense,"OSS","")
	
	
	newcommand(paste0("acceptanceRateTable",outsider,restrict),
		print(resultTable,
			hline.after=c(0),
			sanitize.text.function = bold,
			sanitize.colnames.function = bold,		
			include.rownames=FALSE,
			print.results=FALSE,
			comment=FALSE))

	chisq = chisq.test(rbind(gender1row,gender2row))
	newcommand(paste0("acceptanceRateChiSq",outsider,restrict),
					chiSqFormat(chisq))

	if(!excludeInsiders & !restrictLicense){
	
		library("compute.es")
	
		rate <- function(res,gender){
			as.numeric(sub("%", "", res[which(res$Gender==gender),]$`Merge Rate`))
		}
		
		diff <- rate(mainResult,"Women")-rate(mainResult,"Men")
		
		newcommandPercent("overallDiff",diff/100)
		newcommand("acceptanceRateR",
				paste0('$r=',						
				gsub('0\\.','.',chies(chisq$statistic,sum(gender1row)+sum(gender2row),verbose=FALSE)$r),
				'$'))

		sumCols <- function(col){
			sum(as.numeric(gsub(',','',col)))
		}
		
		newcommandPercent('totalAcceptance',sumCols(mainResult$Merged)/
														(sumCols(mainResult$Open) +
														 sumCols(mainResult$Closed) +
														 sumCols(mainResult$Merged)))
	}
	
	
	return(mainResult)
}


overallAnalysis <- function(con){

	notes("Total pull request acceptance rate")
	commonAnalysis(con)
	
	notes("Total outsider pull request acceptance rate")
	insiderResult <- commonAnalysis(con,excludeInsiders=TRUE);
	
	newCommandForMergeRate <- function(result,gender){
		rate <- subset(result,Gender==gender,)$`Merge Rate`
		ci <-   subset(result,Gender==gender,)$`95% Confidence Interval`
		newcommand(paste0("acceptanceRate",gender,"Insiders"),
				gsub("%","\\%",paste(rate,ci),fixed=TRUE))
	}
		
	newCommandForMergeRate(insiderResult,"Men")
	newCommandForMergeRate(insiderResult,"Women")
	
	notes("Acceptance rates for open source")
	commonAnalysis(con,restrictLicense=TRUE)
	
	return(insiderResult)
}