library(plyr)

projectAnalysis <- function(con){
        tempdir()
	d <- dbGetQuery(con,"select gender,repos,count(creator_id) as users,avg(rate) as accept from
					(SELECT creator_id,gender,avg(mgd) as rate, count(distinct base_repo_id) as repos
					FROM
					(SELECT 
						creator_id,
						gender,
						IF(state='merged', 1, 0) mgd,
						base_repo_id
						FROM
						g_pull_closes2 gpc
						LEFT JOIN pull_requests pr on pr.id = gpc.pull_request_id
						WHERE gender in ('Male','Female')
						AND state in ('merged', 'closed', 'open')
					) whatevs GROUP BY creator_id,gender) as whatevs2
					GROUP BY gender,repos")
        		
	d$gender = ifelse(d$gender == 'Female', 'Women', 'Men')		
		
	# add some more columns to data
	d$x <- d$repos
	d$y <- d$accept
	
	d$confidence <- binom.confint(d$users * d$accept, d$users, 0.95, "exact");
	d$lower <- pmax(minY(),d$confidence$lower);
	d$upper <- pmin(maxY(),d$confidence$upper);
	d$range <- d$upper - d$lower;
	
	#data males, data females
	dm <- d[which(d$gender=='Men'),];
	df <- d[which(d$gender=='Women'),];

	topIndex <- 10;
	maleColour <- "#660066";
	femaleColour <- "#ffbf80";
	ciAlpha <- 0.5;
	
	result <- ggplot(data=d[0:0,], aes(x=x, y=y, colour=gender, fill=gender)) +
			geom_point(data=dm, size=1.0, stat="identity") +
			geom_ribbon(data=dm, aes(ymin=lower, ymax=upper), stat="identity", alpha=ciAlpha, linetype = "blank") +
			geom_point(data=df, size=1.0, stat="identity") +
			geom_ribbon(data=df, aes(ymin=lower, ymax=upper), stat="identity", alpha=ciAlpha, linetype = "blank") +
			scale_colour_manual(values=c(maleColour, femaleColour),
					breaks=c("Women", "Men"), name='') +
			scale_fill_manual(values=c(maleColour, femaleColour),
					breaks=c("Women", "Men"), name='') +
			y_ar_scale() +
			scale_x_continuous(expand = c(0,0), limit=c(1, topIndex), 
					breaks=c(1:topIndex)) +
			xlab("Projects Contributed To") +
			ylab("Acceptance Rate") +
			theme_bw() +
			theme(legend.text = element_text(size = 12), legend.position = c(0.1, 0.9), legend.key = element_rect(fill = rgb(0,0,0,0), color=rgb(0,0,0,0)), legend.background = element_rect(fill = rgb(0,0,0,0))) +
			theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
			theme(panel.grid.minor.x = element_blank()) + 
			theme(panel.border = element_rect(linetype = "blank"))
	
	pdf('Publication - Conference - Gender Bias in Pull Requests/figs/acceptRateByProjectStrata.pdf')
	print(x=result)
	dev.off()
	
	quantiles <- function(gender,percentile){
		d2 <- d[which(d$gender==gender),];
		de <- d2[rep(row.names(d2),d2$users),1:2]
		qs <- ddply(de,c("gender"),function(x) quantile(de$repos,c(percentile)))
		return(qs[,2])
	}
	
	men50th <- quantiles('Men',0.5) 
	women50th <- quantiles('Women',0.5)
	
	if(men50th!=women50th){
		stop("Write other paper narrative here!")
	}
	
	newcommand("projectLevelsSummary",
		paste0("While the median number of projects contributed to via pull request is ", 
				men50th," for both genders (that is, the 50th percentile of developers);
				at the 75th percentile it is ",
					quantiles('Women',0.75)," for women and ",
					quantiles('Men',0.75), " for men, 
				and at the 90th percentile it is ",
					quantiles('Women',0.9)," for women and ",
					quantiles('Men',0.9),  " for men."))
	
	shortened <- d[c('gender','repos','accept','users','lower','upper')][which(d$x<=10),]
	shortened <- shortened[order(shortened$repos),]
	shortTab <- xtable(shortened,auto=TRUE)
	
	newcommand(paste0("projectLevelsTable"),
			print(shortTab,
					hline.after=c(0),		
					include.rownames=FALSE,
					print.results=FALSE,
					comment=FALSE,
					floating=FALSE))
	
	return(d)
}
