require(gridExtra)
require(ggplot2)
library(scales)

acceptanceDistribution <- function(con){
	
query <-  sqlImport("acceptance_distribution_56/first.sql") 

data<-dbGetQuery(con, query)	

data$gender = ifelse(data$gender == 'Female', 'Women', 'Men')

data[data$merge_rate == 1,]$merge_rate = 0.99
data[data$merge_rate == 0,]$merge_rate = 0.01

data$merge_rate = data$merge_rate

fp <- ggplot(data[data$gender == "Women",], aes(x=merge_rate)) +
  geom_histogram(aes(y = ..count.. / sum(..count..)), breaks = seq(0, 1, 0.1), closed = c("right"), fill='#ffbf80', col='white') +
  ggtitle("Women") +
  scale_y_continuous(expand = c(0,0), limits=c(0, 0.401), labels = scales::percent, oob = rescale_none) +
  scale_x_continuous(limits=c(0, 1), labels = scales::percent, oob = rescale_none) +
  ylab("% of Women") +
  xlab("") +
  theme_bw() +
  theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
  theme(panel.border = element_rect(linetype = "blank")) +
  theme(panel.grid = element_line(linetype = "blank"),
        panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
        panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2))

mp <- ggplot(data[data$gender == "Men",], aes(x=merge_rate)) +
  geom_histogram(aes(y = ..count.. / sum(..count..)), breaks = seq(0, 1, 0.1), closed = c("right"), fill='#660066', col='white') +
  ggtitle("Men") +
  scale_y_continuous(expand = c(0,0), limits=c(0, 0.401), labels = scales::percent, oob = rescale_none) +
  scale_x_continuous(limits=c(0, 1), labels = scales::percent, oob = rescale_none) +
  ylab("% of Men") +
  xlab("") +
  theme_bw() +
  theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
  theme(panel.border = element_rect(linetype = "blank")) +
  theme(panel.grid = element_line(linetype = "blank"),
        panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
        panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2))

pdf('Publication - Conference - Gender Bias in Pull Requests/figs/success_distribution.pdf')
grid.arrange(fp, mp, ncol=2, bottom="Acceptance Rate")
dev.off()


newcommand("successDistFig",paste0("\\begin{figure}[t]
			\\centering
			\\includegraphics[width=0.8\\textwidth]{figs/success_distribution}
			\\caption{Histogram of mean acceptance rate per developer for Women 
			(Mean ", percentFormat(mean(data[data$gender == "Women", ]$merge_rate)),",
		  	Median ",percentFormat(median(data[data$gender == "Women", ]$merge_rate)),") and Men 
			(Mean ", percentFormat(mean(data[data$gender == "Men", ]$merge_rate)),", 
			Median ",percentFormat(median(data[data$gender == "Men", ]$merge_rate)),")
			}\\label{fig:success_distribution}
		\\end{figure}"))

makeTable <- function(data){
	resultTable <- xtable(ggplot_build(data)$data[[1]][c('y','xmin','xmax')])
	#align(resultTable) <- "ll|rrrr"
	
	newcommand(paste0("successDistTable",data$labels$title),
			print(resultTable,
					hline.after=c(0),		
					include.rownames=FALSE,
					print.results=FALSE,
					comment=FALSE,
					floating=FALSE))
}

makeTable(fp)
makeTable(mp)

return(data)

}