library(xtable)

codeSizeByGender <- function(con){
	
query <- sqlImport("code_size_22/code_size.sql") 

data <- dbGetQuery(con, query)
# gender, lines_added, lines_deleted, files_changed_count, commit_count, net_lines

females <- data[data$gender == "Female", ]
males   <- data[data$gender == "Male", ]

test.add <- t.test(females[,2],males[,2],conf.int=TRUE)
test.del <- t.test(females[,3],males[,3],conf.int=TRUE)
test.cng <- t.test(females[,4],males[,4],conf.int=TRUE)
test.cmt <- t.test(females[,5],males[,5],conf.int=TRUE)
test.net <- t.test(females[,6],males[,6],conf.int=TRUE)


ci <- function(interval){
	return(paste0("[",numberFormat(interval[[1]],1),",",numberFormat(interval[[2]],digits=1),"]"))
}

p <- function(val){
	return(paste0("$",pValFormat(val),"$"))
}

newcommand("genderCommitsChangedTable",
	paste0("\\begin{tabular}{ll|rr|r|r}
 & & lines added & lines removed & files changed & commits\\\\
\\hline
Women  & median &   ",numberFormat(median(females[,2]))," &   ",numberFormat(median(females[,3])), "& ",numberFormat(median(females[,4])),"   &  ",numberFormat(median(females[,5])),"\\\\
		& mean   & ",numberFormat(mean(females[,2]))," & ",numberFormat(mean(females[,3]))," & ",numberFormat(mean(females[,4]),1)," & ",numberFormat(mean(females[,5]),1),"\\\\
\\hline
Men    & median &   ",numberFormat(median(males[,2]))," &   ",numberFormat(median(males[,3]))," & ",numberFormat(median(males[,4])),"   &  ",numberFormat(median(males[,5])),"\\\\
	    & mean   &  ",numberFormat(mean(males[,2]))," & ",numberFormat(mean(males[,3]))," & ",numberFormat(mean(males[,4]),1)," & ",numberFormat(mean(males[,5]),1),"\\\\
\\hline
t-test & statistic &",numberFormat(test.add$statistic,2),"&",numberFormat(test.del$statistic,2),"&",numberFormat(test.cng$statistic,2),"&",numberFormat(test.cmt$statistic,2),"\\\\
		& df	& 	",numberFormat(test.add$parameter),"&",numberFormat(test.del$parameter),"&",numberFormat(test.cng$parameter),"&",numberFormat(test.cmt$parameter),"\\\\
		& p		& 	",p(test.add$p.value),"&",p(test.del$p.value),"&",p(test.cng$p.value),"&",p(test.cmt$p.value),"\\\\
		& CI	& 	",ci(test.add$conf.int),"&",ci(test.del$conf.int),"&",ci(test.cng$conf.int),"&",ci(test.cmt$conf.int),"\\\\
\\end{tabular}"
))


newcommand("genderNetLinesChangedTable",
	paste0("\\begin{tabular}{ll|rr|r|r}
			& & net lines changed \\\\
			\\hline
			women  & median &",   numberFormat(median(females[,6])),"\\\\
				& mean   & ",numberFormat(mean(females[,6])),"\\\\
			\\hline
			men    & median &   ",numberFormat(median(males[,6])),"\\\\
	    		& mean   &  ",numberFormat(mean(males[,6])),"\\\\
\\hline
t-test & statistic &",numberFormat(test.net$statistic,2),"\\\\
		& df	& 	",numberFormat(test.net$parameter),"\\\\
		& p		& 	",p(test.net$p.value),"\\\\
		& CI	& 	",ci(test.net$conf.int),"\\\\
\\end{tabular}\n"
			
			))


newcommand("genderSizeN", numberFormat(nrow(females) + nrow(males)))
}
