-- associate the pull acceptace rate per user per pull
-- the pull acceptance rate at pull x is the average
delimiter //
drop procedure if exists associate_creator_pull_acceptance_rate;
create procedure associate_creator_pull_acceptance_rate(external bool)
begin

  declare finished bool default false;

  -- cursor variables
  declare userId int;
  declare state varchar(255);
  declare pullReqId int;

  -- variables for determining accptance rate per user
  declare currentUserId int default -1;
  declare totalPulls int;
  declare mergedPulls int;

  declare cur_ext cursor for
  select pc.creator_id, pc.state, pc.pull_request_id
  from g_pull_closes pc
  where pc.gplus_gender is not null
    and pc.creator_external_pull_index is not null
  order by pc.creator_id asc, pc.creator_external_pull_index asc;

  declare cur_all cursor for
  select pc.creator_id, pc.state, pc.pull_request_id
  from g_pull_closes pc
  where pc.gplus_gender is not null
    and pc.creator_pull_index is not null
  order by pc.creator_id asc, pc.creator_pull_index asc;

  declare continue handler
  for not found set finished = true;

  if(external) then
    open cur_ext;
  else
    open cur_all;
  end if;

  theLoop: loop

    if(external) then
      fetch cur_ext into userId, state, pullReqId;
    else
      fetch cur_all into userId, state, pullReqId;
    end if;

    if(finished) then
      leave theLoop;
    end if;

    if(userId <> currentUserId) then
      set currentUserId = userId;
      set totalPulls = 0;
      set mergedPulls = 0;
    end if;

    if(state in ('merged', 'open', 'closed')) then
      set totalPulls = totalPulls + 1;

      if(state = 'merged') then
        set mergedPulls = mergedPulls + 1;
      end if;

      if (external) then
        update g_pull_closes pc
        set pc.creator_external_pull_acceptance_rate = mergedPulls / totalPulls
        where pc.pull_request_id = pullReqId;
      else
        update g_pull_closes pc
        set pc.creator_pull_acceptance_rate = mergedPulls / totalPulls
        where pc.pull_request_id = pullReqId;
      end if;

    end if;

  end loop theLoop;

end //
delimiter ;

call associate_creator_pull_acceptance_rate(false);
call associate_creator_pull_acceptance_rate(true);

drop procedure associate_creator_pull_acceptance_rate;
