delimiter //
drop procedure if exists associate_creator_pull_index;
create procedure associate_creator_pull_index(external bool)
begin
  declare finished bool default false;
  declare userId int;

  -- all users with pulls that haven't been indexed
  declare cur_userIds_ext cursor for
  select distinct pc.creator_id
  from g_pull_closes pc
  where creator_external_pull_index is null
    and pc.gplus_gender in ('Male', 'Female')
    and pc.creator_type = '';

  declare cur_userIds_all cursor for
  select distinct pc.creator_id
  from g_pull_closes pc
  where creator_pull_index is null
    and pc.gplus_gender in ('Male', 'Female');

  declare continue handler
  for not found set finished = true;

  if(external) then
    open cur_userIds_ext;
  else
    open cur_userIds_all;
  end if;

  theLoop: loop

    if(external) then
      fetch cur_userIds_ext into userId;
    else
      fetch cur_userIds_all into userId;
    end if;

    if(finished = true) then
      leave theLoop;
    end if;

    begin
      call associate_creator_pull_index_(external, userId);
    end;

  end loop theLoop;

end//
delimiter ;

delimiter //
drop procedure if exists associate_creator_pull_index_;
create procedure associate_creator_pull_index_(external bool, userId int)
begin
  declare finished bool default false;
  declare pullReqId int;
  declare idx int default 0;

  -- all pulls for this user 
  declare cur_indexes_ext cursor for
  select gp.pull_request_id
  from g_pull_closes gp
  join issues i on i.pull_request_id = gp.pull_request_id
  where gp.creator_id = userId
    and gp.creator_type = ''
  group by gp.pull_request_id
  order by max(i.created_at) asc;

  declare cur_indexes_all cursor for
  select gp.pull_request_id
  from g_pull_closes gp
  join issues i on i.pull_request_id = gp.pull_request_id
  where gp.creator_id = userId
  group by gp.pull_request_id
  order by max(i.created_at) asc;

  declare continue handler
  for not found set finished = true;

  if(external) then
    open cur_indexes_ext;
  else
    open cur_indexes_all;
  end if;

  theLoop: loop

    if(external) then
      fetch cur_indexes_ext into pullReqId;
    else
      fetch cur_indexes_all into pullReqId;
    end if;

    if(finished = true) then
      leave theLoop;
    end if;

    set idx = idx + 1;

    if(external) then
      update g_pull_closes
      set creator_external_pull_index = idx
      where pull_request_id = pullReqId;
    else
      update g_pull_closes
      set creator_pull_index = idx
      where pull_request_id = pullReqId;
    end if;

  end loop theLoop;

end//
delimiter ;

call associate_creator_pull_index(false);
call associate_creator_pull_index(true);

drop procedure associate_creator_pull_index;
