require(binom)
library(ggplot2);

rateOverTime <- function(con){

# plotting variables
topIndex <- 128;
maleColour <- "#660066";
femaleColour <- "#ffbf80";
criticalValue <- 1.645;
ciAlpha <- 0.5;

query <-  sqlImport("g_results/graph_acceptance_rate_vs_pull_request_count.sql") 
d <-dbGetQuery(con, query)

d$gender = ifelse(d$gender == 'Female', 'Women', 'Men')

# add some more columns to data
d$x <- d$index
d$y <- d$avg_acceptance_rate

d$confidence <- binom.confint(d$users * d$avg_acceptance_rate, d$users, 0.95, "exact");
d$lower <- d$confidence$lower;
d$upper <- d$confidence$upper;
d$range <- d$upper - d$lower;

#data males, data females
dm <- d[(1:topIndex*2),];
df <- d[(1:topIndex*2)-1,];

# create a model of data for graphing it
pm <- smooth.spline(x=dm$x, y=dm$y, w=1/dm$range);
dm$py <- data.frame(predict(pm, dm$x))$y;

pf <- smooth.spline(x=df$x, y=df$y, w=1/df$range);
df$py <- data.frame(predict(pf, df$x))$y;

result <- ggplot(data=d[0:0,], aes(x=x, y=y, colour=gender, fill=gender)) +
  geom_point(data=dm, size=1.0, stat="identity") +
  geom_ribbon(data=dm, aes(ymin=lower, ymax=upper), stat="identity", alpha=ciAlpha) +
  geom_point(data=df, size=1.0, stat="identity") +
  geom_ribbon(data=df, aes(ymin=lower, ymax=upper), stat="identity", alpha=ciAlpha) +
  scale_colour_manual(values=c(maleColour, femaleColour),
      breaks=c("Women", "Men"), name='') +
  scale_fill_manual(values=c(maleColour, femaleColour),
      breaks=c("Women", "Men"), name='') +
  y_ar_scale() +
  scale_x_continuous(expand = c(0,0), limit=c(1, topIndex), trans="log2",
      breaks=c(1, 2, 4, 8, 16, 32, 64, 128, 300)) +
  xlab("Pull Requests") +
  ylab("Acceptance Rate") +
  theme_bw() +
  theme(legend.text = element_text(size = 12), legend.position = c(0.1, 0.9), legend.key = element_rect(fill = rgb(0,0,0,0), color=rgb(0,0,0,0)), legend.background = element_rect(fill = rgb(0,0,0,0))) +
  theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
  theme(panel.border = element_rect(linetype = "blank"))

pdf('Publication - Conference - Gender Bias in Pull Requests/figs/accepts_over_time.pdf')
print(x=result)
dev.off()

newcommand("exampleRateFluct",
		paste0("For instance, at 128 pull requests, only ",
		d[which(d$index==128 & d$gender=='Women'),'users'],
		" women are represented."))

am <- dm[which(dm$index %in% c(16,32)),]$avg_acceptance_rate
newcommandPercent("menIncreaseStep",max(am)-min(am));

shortened <- d[c('gender','index','avg_acceptance_rate','users','lower','upper')][which(d$index<=128),]
shortTab <- xtable(shortened,auto=TRUE)

newcommand(paste0("timeTable"),
		print(shortTab,
				hline.after=c(0),		
				include.rownames=FALSE,
				print.results=FALSE,
				comment=FALSE,
				floating=FALSE,
				tabular.environment = "longtable"))

return(d)
}
