-- One time for efficiency:
-- ALTER TABLE g_pull_closes2 ADD INDEX pull_request_id (pull_request_id);
-- ALTER TABLE g_pull_closes2 ADD INDEX repo_id (repo_id);
-- ALTER TABLE issue_events ADD INDEX created_at (created_at);
-- ALTER TABLE issue_events ADD INDEX action (action);

DROP TABLE IF EXISTS openings;
CREATE TEMPORARY TABLE openings AS
(
SELECT pull_hist.pull_request_id, base_repo_id, actor_id opener, created_at 
		FROM (SELECT creator_id, pull_request_id FROM g_pull_closes2 WHERE /*pull_request_id<7000 AND*/ creator_type='Outsider') gpc
        JOIN sandbox.pull_hist ON creator_id=actor_id and gpc.pull_request_id=pull_hist.pull_request_id
        WHERE action='opened');
ALTER TABLE openings ADD INDEX idx (pull_request_id,opener,base_repo_id);



DROP TABLE IF EXISTS issuejudgements_before_pr;
CREATE TEMPORARY TABLE issuejudgements_before_pr AS
(SELECT pull_request_id,kind,MIN(earlier) earlier FROM
(SELECT openings.pull_request_id,"issue" kind,issues.issue_id earlier FROM 
	openings JOIN issues ON openings.base_repo_id=issues.repo_id 
			 JOIN issue_events events ON issues.id=events.issue_id
    WHERE
		events.action='closed' -- for an issue that was closed
        AND issues.pull_request_id IS NULL -- issue is not a PR
        AND openings.opener = events.actor_id
        AND openings.opener != issues.reporter_id -- where pull requestor was not the issue closer
 		AND openings.created_at >= events.created_at -- and PR happened at the same time or after the issue
        AND (openings.created_at - INTERVAL 90 DAY) < events.created_at -- but were within 3 months of each other
)
AS x GROUP BY pull_request_id,kind)
;

DROP TABLE IF EXISTS prjudgements_before_pr;
CREATE TEMPORARY TABLE prjudgements_before_pr AS
(SELECT pull_request_id,kind,MIN(earlier) earlier FROM
(SELECT openings.pull_request_id,"pull" kind,gpc.pull_request_id earlier FROM 
	openings JOIN g_pull_closes2 gpc ON openings.base_repo_id=gpc.repo_id AND openings.opener=gpc.merger_closer_id
			 JOIN sandbox.pull_hist ph ON gpc.pull_request_id = ph.pull_request_id AND openings.opener=ph.actor_id
	WHERE    
		((gpc.state='merged' AND ph.action in ('merged','closed'))-- if the opener of PR-1 merged PR-2...
        OR (ph.action in ('merged','closed') -- or closed/merged a PR-2 that....
        AND ph.pull_request_id IN 
			(SELECT pull_request_id FROM pull_request_history prh2 WHERE 
						prh2.pull_request_id=ph.pull_request_id AND
                        prh2.action='opened' AND
                        prh2.actor_id!=ph.actor_id))) -- they didn't open
		AND openings.created_at >= ph.created_at -- and PR-1 happened at the same time or after PR-2
        AND (openings.created_at - INTERVAL 90 DAY) < ph.created_at -- and but were within 3 months of each other
)
AS x GROUP BY pull_request_id,kind)
;

DROP TABLE IF EXISTS sandbox.ejected_insiders;
CREATE TABLE sandbox.ejected_insiders AS
(SELECT * FROM issuejudgements_before_pr)
UNION ALL
(SELECT * FROM prjudgements_before_pr);

INSERT INTO sandbox.ejected_insiders
(SELECT pull_request_id,"pull",pull_request_id FROM g_pull_closes2 WHERE creator_type='Outsider' AND self_merge=1);

ALTER TABLE sandbox.ejected_insiders ADD INDEX pull_request_id (pull_request_id);

SELECT pull_request_url(pull_request_id),kind,if(kind='pull',pull_request_url(earlier),CAST(earlier AS CHAR)) FROM sandbox.ejected_insiders;