import MySQLdb
import os
import threading
import sys
dirpath = os.path.dirname(os.path.realpath(__file__))
gcPath = dirpath + '/../submodules/genderComputer'
sys.path.append(gcPath)
from genderComputer import GenderComputer

class GenderAssociator:
    def __init__(self):
        self.inited = False

    def init(self):
        if self.inited:
            return

        self.gc = GenderComputer(os.path.abspath(gcPath + '/nameLists'))

        self.db = MySQLdb.connect(
            host    = os.environ['MYSQL_HOST'],
            user    = os.environ['MYSQL_USER'],
            passwd  = os.environ['MYSQL_PASSWORD'],
            db      = os.environ['MYSQL_DATABASE'],
            port    = int(os.environ['MYSQL_PORT']),
        )

        self.cur = self.db.cursor()

    def run(self):
        self.init()

        while True:
            rows = self.nextUsers()

            if len(rows) == 0:
                break;

            self.assignGenders(rows)

    def nextUsers(self):
        self.cur.execute('''
            select userId, login, name
            from g_user_predicted_genders
            where login_gender is null
              and userId >= 7000000
            order by userId asc
            limit 10000
        ''')

        return self.cur.fetchall()

    def resolveGender(self, name):
        if name == None or name == '':
            return None

        try:
            gender = self.gc.resolveGender(unicode(name), None)
        except UnicodeDecodeError as e:
            return None

        if gender == 'name not found' or gender == 'error in name':
            gender = 'unknown'
        else:
            gender = gender[len('is '):]
            if gender[0] == 'u':
                gender = 'unisex' 

        return gender

    def encodeGenderForDb(self, gender):
        return 'null' if gender == None else "'%s'" % gender

    def assignGenders(self, users):
        for user in users:
            # print '"%s"' % user[1], '"%s"' % user[2]
            login_gender = self.resolveGender(user[1])
            name_gender = self.resolveGender(user[2])
            assert login_gender <> None
            params = (
                self.encodeGenderForDb(login_gender),
                self.encodeGenderForDb(name_gender),
                int(user[0]),
            )
            # print login_gender, name_gender
            query = '''
                update g_user_predicted_genders
                set login_gender = %s,
                    name_gender = %s
                where userId = %s
            ''' % params
            self.cur.execute(query)

if __name__ == '__main__':
    ga = GenderAssociator()
    ga.run()
