var mysql = require('lib/mysql'),
    _ = require('lodash'),
    Promise = require('promise'),
    fs = require('fs'),
    assert = require('assert'),
    csv = require('fast-csv');

if (process.argv.length !== 3) {
  console.log('usage: node load_demograhics.js <demographics.csv>');
  process.exit();
}

var stream = fs.createReadStream(process.argv[2]);
csv.fromStream(stream, {delimiter: ';', headers: true})
//.transform(function(data){
//   return data;
//})
.on('data', function(data){
  addUser(data);
})
.on('end', function(){
  finalize();
});

var users = [];
function addUser(demographicsRow) {
  var userIds = demographicsRow.user_ids.split(',');
  var gender = demographicsRow.gender;
  assert(['male', 'female', ''].indexOf(gender) !== -1, demographicsRow);

  for (var i = 0; i < userIds.length; ++i) {
    if (userIds[i] === '') {
      continue;
    }
    users.push([userIds[i], gender || null]);
  };
  if (users.length >= 1000) {
    addBatch();
  }
}

function finalize() {
  addBatch();
  Promise.all(batches).then(function() {
    mysql.connection.end();
    console.log('done');
  }).done();
}

var batches = [];
var count = 0;
function addBatch() {
  var toAdd = users;
  users = [];

  if (toAdd.length) {
    var p = mysql.promiseQuery('insert into g_users (id, gender) values ?',
        [toAdd])
    .then(function(res) {
      count += toAdd.length;
      console.log('added: ' + count);
      assert.strictEqual(res.affectedRows, toAdd.length);
    });
    batches.push(p);
  }
}
