var Promise = require('promise'),
    _ = require('lodash'),
    promiseRawPull = require('lib/promiseRawPull'),
    parseRawPull = require('lib/parseRawPull')
    realDebug = require('debug')('child');


function debug() {
  return;
  var args = Array.prototype.slice.call(arguments);
  var message = args.shift();
  args.unshift(id);
  args.unshift('(%s) ' + message);
  realDebug.apply(null, args);
}

var troublesomePullIds = null,
    completeCount = 0,
    maxActive = process.env.PULLS_ACTIVE_PER_WORKER || 200,
    maxComplete = process.env.PULLS_COMPLETE_PER_WORKER || 1000,
    requestingIds = false,
    stopping = false,
    restarting = false,
    active = [],
    completeUpTo = null,
    callback = null,
    id = null;

process.on('message', onMessage);
var updateInterval = setInterval(update, 1000);

function update() {
  sendStatus();
  checkDone();
}

function sendStatus(exit) {
  process.send({
    code: exit ? 'exit' : 'status',
    completions: completeCount,
    completeUpTo: completeUpTo,
  });
}

function onMessage(message) {
  switch (message.code) {
  case 'troublesomePullId':
    onTroublesomePullId(message);
    break;
  case 'stop':
    onStop(message);
    break;
  case 'init':
    onInit(message);
    break;
  case 'ids':
    onIds(message);
    break;
  default:
    throw new Error('unrecognized message code: ' + message.code);
  };
  debug('message %s', message.code);
}

function onTroublesomePullId(message) {
  troublesomePullIds.push(message.id);
}

function onStop() {
  stopping = true;
  checkDone();
}

function checkDone() {
  if (active.length === 0 && (stopping || restarting)) {
    clearInterval(updateInterval);
    sendStatus(true);
    if (stopping) {
      process.send({code: 'stop'});
      debug('stop');
    } else {
      process.send({code: 'restart'});
      debug('restart');
    }
    process.removeListener('messsage', onMessage);
  }
}

function onInit(message) {
  callback = require(message.callbackPath);

  troublesomePullIds = message.troublesomePullIds;
  promiseRawPull.setTroublesomePullIds(troublesomePullIds);

  requestingIds = true;
  process.send({code: 'ids', count: maxActive});

  id = message.id;
}

function onIds(message) {
  requestingIds = false;
  debug("ids %o", message.ids);

  _.each(message.ids, function(id) {
    promiseRawPull(id)
    .then(parseRawPull)
    .then(function(pull) {
      if (pull) {
        return callback(pull);
      }
    })
    .then(function() {

      ++completeCount;

      var idx = active.indexOf(id);
      if (idx === 0) {
        completeUpTo = id;
      }
      active.splice(idx, 1);

      if (!requestingIds && !stopping && !restarting) {
        requestIds();
      }
    })
    .done();
  });
  active = active.concat(message.ids);
}

function requestIds() {
  var count = maxActive - active.length;
  var maxCountRemaining = maxComplete - (completeCount + active.length);
  var count = Math.min(count, maxCountRemaining);
  if (count < 0) count = 0;
  if (count === 0) {
    restarting = true;
  } else {
    requestingIds = true;
    process.send({code: 'ids', count: count});
  }
}
