var Promise = require('promise'),
    _ = require('lodash'),
    realDebug = require('debug')('IterateOverThings:Worker');

module.exports = finishThing

process.on('uncaughtException', function(e) {
  e._message = e.message;
  e._stack = e.stack;
  process.send({code: 'error', error: e});
  process.exit(-1);
});

function debug() {
  return; // comment this out to debug in children
  var args = Array.prototype.slice.call(arguments);
  var message = args.shift();
  args.unshift(id);
  args.unshift('(%s) ' + message);
  realDebug.apply(null, args);
}

var completeCount = 0,
    maxActive = process.env.maxActiveThingsPerWorker || 200,
    maxComplete = process.env.maxCompleteThingsPerWorker || 1000,
    stopping = false,
    restarting = false,
    activeCount = 0,
    workerModule = null,
    requestingThings = false;

process.on('message', onMessage);
var updateInterval = setInterval(update, 1000);

function update() {
  sendStatus();
  checkDone();
}

function sendStatus(exit) {
  process.send({
    code: exit ? 'exit' : 'status',
    completions: completeCount,
  });
}

function onMessage(message) {
  switch (message.code) {
  case 'stop':
    onStop(message);
    break;
  case 'init':
    onInit(message);
    break;
  case 'things':
    onThings(message);
    break;
  default:
    throw new Error('unrecognized message code: ' + message.code);
  };

  debug('message %s', message.code);
}

function onStop() {
  stopping = true;
  checkDone();
}

function checkDone() {
  if (activeCount === 0 && (stopping || restarting)) {
    clearInterval(updateInterval);
    sendStatus(true);

    Promise.resolve().then(
      (workerModule.pStop ? workerModule.pStop : Promise.resolve)
    )
    .done(function() {
      if (stopping) {
        process.send({code: 'stop'});
        debug('stop');
      } else {
        process.send({code: 'restart'});
        debug('restart');
      }
      process.removeListener('message', onMessage);
    })
  }
}

function onInit(message) {
  id = message.id;
  workerModule = require(message.workerPath);

  Promise.resolve()
  .then(function() {
    return workerModule.pInit ?
      workerModule.pInit(message.id, message.msg) : Promise.resolve();
  })
  .done(function() {
    requestingThings = true;
    process.send({code: 'things', count: maxActive});
  });
}

function onThings(message) {
  requestingThings = false;
  debug("things %o", message.things);

  _.each(message.things, function(thing) {
	activeCount += 1; //message.things.length;
	Promise.resolve(thing)
		.then(workerModule.pProcess) //TO how-old.net/Worker.js
		.catch(function(e) {
			console.log("Error caught in onThings: " + e.message)
		   	//onThings({code: "things", things: thing}) //Still in testing.
		})
		.done();
  });
}

function finishThing() {
	  ++completeCount;
	  --activeCount;
	  console.log("\t" + completeCount + "\t" + activeCount);
	  if (!requestingThings && !stopping && !restarting) {
	    requestThings();
	  }
}

function requestThings() {
  var count = maxActive - activeCount;
  var maxCountRemaining = maxComplete - (completeCount + activeCount);
  var count = Math.min(count, maxCountRemaining);
  if (count < 0) count = 0;
  if (count === 0) {
    restarting = true;
  } else {
    requestingThings = true;
    process.send({code: 'things', count: count});
  }
}
