/**
 * return the pull_request_object if this html represents a valid pull
 * send the parent process 'troublesomePullId' if this is a troublesome pull
 * ignore errors and non-existent files
 */
var _ = require('lodash'),
    PullRequest = require('lib/pull_request_dom/pull_request');

module.exports = parseRawPull;

/**
 *
 */
function parseRawPull(raw) {
  switch (raw.type) {
  case 'troublesome':
  case 'error':
  case 'no_exist':
    break;
  case 'html':
    if (pullIsUnavailable(raw.html)) {
      return;
    } else if (!isPullRequest(raw.html)) {
      process.send({code: 'troublesomePullId', id: raw.id});
      return;
    } else {
      return new PullRequest(raw.html, raw.id);
    }
    break;
  default:
    throw new Error('unrecognized raw-pull type: ' + raw.type);
  }
}

function pullIsUnavailable(html) {
  return containsSignificantText(html, significantText.error);
}

function isPullRequest(html) {
  return containsSignificantText(html, significantText.pullRequest);
}

function containsSignificantText(html, texts) {
  return _.some(texts, _.contains.bind(_, html));
}

var significantText = {
  error: [
    'class="mega-octicon octicon-alert',
    "class='mega-octicon octicon-alert",
  ],
  pullRequest: [
    'class="view-pull-request',
    "class='view-pull-request",
  ],
};
