var Https = require('https'),
    Promise = require('promise'),
    Url = require('url'),
    Buffer = require('buffer');

module.exports = promiseHtml;

/**
 * Resolve to the html of the url.
 * If status code != 200, reject to error with error.code = status code.
 */
function promiseHtml(url) {
  return new Promise(function(resolve, reject) {
    var req = Https.request(url, onResponse);
    req.end();
    req.on('error', onError);

    function onResponse(res) {
      if (res.statusCode !== 200) {
        var e = new Error('Status not okay: ' + res.statusCode);
        e.code = res.statusCode;
        e.res = res;
        reject(e);
      } else {
        res.on('data', onData);
        res.on('end', onEnd);
      }
      var body = '';
      function onData(chunk) {
        body += chunk;
      }
      function onEnd() {
        // to utf-8...escape pesky utf16 chars
        // ecmanaut.blogspot.com/2006/07/encoding-decoding-utf8-in-javascript.html
        resolve(unescape(encodeURIComponent(body)));
      }
    }

    function onError(e) {
      reject(e);
    }
  });
}
