/**
 * Promise the raw pull from disk.
 *   - users must initialize the troublesomePullIds array
 *   - returns an object with unparsed pull data
 */
module.exports = promisePull;
promisePull.setTroublesomePullIds = setTroublesomePullIds;

var Promise = require('promise'),
    fs = require('fs'),
    promiseReadFile = Promise.denodeify(fs.readFile.bind(fs)),
    pullsPath = process.env.PULLS_PATH || '/mnt/flashdrive/pulls/';


/**
 * Set reference to troublesomePullIds array.
 */
var troublesomePullIds = null;
function setTroublesomePullIds(ids) {
  troublesomePullIds = ids;
}

/**
 * Read the pull request from disk and resolve to:
 * {
 *    type: troublesome/error/html/no_exist
 *    id: pulls' id
 *    code: (if type=error)
 *    html: (if type=html)
 * }
 */
function promisePull(pullRequestId) {
  if (troublesomePullIds.indexOf(pullRequestId) !== -1) {
    return Promise.resolve({
      type: 'troublesome',
      id: pullRequestId,
    });
  }
  return new Promise(function(resolve, reject) {
    var path = pullsPath + pullRequestId;
    fs.exists(path + '.err', errExists);
    function errExists(exists) {
      if (exists) {
        var p = promiseReadFile(path + '.err').then(function(errorText) {
          return {
            type: 'error',
            code: errorText,
            id: pullRequestId,
          };
        });
        resolve(p);
      } else {
        fs.exists(pullsPath + pullRequestId + '.html', htmlExists);
      }

      function htmlExists(exists) {
        if (exists) {
          var p = promiseReadFile(path + '.html').then(function(html) {
            return {
              type: 'html',
              html: html.toString(),
              id: pullRequestId,
            };
          });
          resolve(p);
        } else {
          resolve({
            type: 'no_exist',
            id: pullRequestId,
          });
        }
      }
    }
  });
}
