var PullRequest = (function definePullRequest() {
  var _ = require('lodash');

  function PullRequest(raw_html, id) {
    var cheerio = require('cheerio');

    this.id = id;
    this.$ = cheerio.load(raw_html); // DOM Object
  }

  PullRequest.prototype.getState = function getState() {
    return this.$('div.state').attr('class').split(' ')[1]
        .substring('state-'.length);
  };

  // PRs that are referenced by this one
  PullRequest.prototype.getReferencesIn = function getReferencesIn() {
    return _.uniq(
        this.$('.title-link .issue-num').map(function(i, e) {
          return this.$(e).parent().attr('href');
        }.bind(this)).toArray()
    );
  };

  // PRs that reference this one
  PullRequest.prototype.getReferencesOut = function getReferencesOut() {
    return _.uniq(
        this.$('.issue-link').map(function(i, e) {
          return this.$(e).attr('href');
        }.bind(this)).toArray()
    );
  };

  PullRequest.prototype.getCreator = function getCreator() {
    var c = this.getComments()[0];
    return {
      creator: c.author,
      creatorStatus: c.authorStatus,
    };
  };

  PullRequest.prototype.getComments = function getComments() {
    var htmlComments = this.$('.comment');
    var comments = [];

    for (var i = 0; i < htmlComments.length; ++i) {
      var htmlComment = htmlComments.eq(i);
      // looks like bodies like this are created in javascript?
      // <p>Nothing to preview</p>
      var body = htmlComment.find('.comment-body').html().trim();
      var author = htmlComment.find('.author').text();
      var authorStatus = htmlComment.find('.timeline-comment-label').text();
      if (body === '<p class="text-muted">\n            ' +
          '<em>No description provided.</em>\n          </p>') {
        body = '';
      }

      comments.push({
        body: body,
        author: author,
        authorStatus: authorStatus,
      });
    }
    return comments;
  };

  PullRequest.prototype.getLabels = function getLabels() {
    return this.$('a.label').map(
        function(i,e) {return this.$(e).attr('title')}.bind(this)).toArray();
  };

  PullRequest.prototype.getText = function getText(index, element) {
    return this.$(element).text();
  };

  PullRequest.prototype.getMerger = function getMerger() {
    return this.$('.discussion-item-merged').last().find('.author').text()
  };

  PullRequest.prototype.getCloser = function getCloser() {
    return this.$('.discussion-item-closed').last().find('.author').text()
  };

  PullRequest.prototype.getMergerCloser = function getMergerCloser() {
    return this.$('.discussion-item-closed, .discussion-item-merged').last()
        .find('.author').text()
  };

  PullRequest.prototype.getLinesAddedCount = function getLinesAddedCount() {
    return parseCount(this.$('#diffstat > .text-diff-added').text().trim()
        .substring(1));
  };

  PullRequest.prototype.getLinesDeletedCount = function getLinesDeletedCount() {
    return parseCount(this.$('#diffstat > .text-diff-deleted').text().trim()
        .substring(1));
  };

  function parseCount(text) {
    var count = text.replace(/[^0-9]/g, '');
    return count === '' ? 0 : parseInt(count);
  }

  PullRequest.prototype.getCommitCount = function getCommitCount() {
    return parseCount(this.$('#commits_tab_counter').text().trim());
  };

  PullRequest.prototype.getFilesChangedCount = function getFilesChangedCount() {
    return parseCount(this.$('#files_tab_counter').text().trim());
  };

  return PullRequest;
})();

if (!module.parent) {
  var raw_html = "";

  process.stdin.on('data', function(data) {
    raw_html += data;
  });

  process.stdin.on('end', function() {
    test_pr = new PullRequest(raw_html);

    process.env.DEBUG = "pull_request";
    var debug = require('debug')('pull_request');

    debug('State: %s', test_pr.getState());
    debug('Labels: %o', test_pr.getLabels());
    debug('Creator: %o', test_pr.getCreator());
    debug('User who Merged this PR: %s', test_pr.getMerger());
    debug('User who Closed this PR: %s', test_pr.getCloser());
    debug('References out: %o', test_pr.getReferencesOut());
    debug('Refrences in: %o', test_pr.getReferencesIn());
    debug('lines added: %s', test_pr.getLinesAddedCount());
    debug('lines deleted: %s', test_pr.getLinesDeletedCount());
    debug('commits: %s', test_pr.getCommitCount());
    debug('files changed: %s', test_pr.getFilesChangedCount());
    debug('Comments: %o', test_pr.getComments());
  });
} else {
  module.exports = PullRequest;
}
