/**
 * This round of parsing inserts:
 * - references to other issues (g_pull_references)
 * - pull labels
 */
if (module.parent) {
  module.exports = roundTwo;

  var mysql = require('lib/mysql'),
      _ = require('lodash'),
      util = require('util'),
      assert = require('assert');

  function roundTwo(pull) {
    return Promise.all([
        pInsertLabels(pull),
        pInsertReferences(pull),
    ]);
  }

  function pInsertReferences(pull) {
    return Promise.all([
      pInsert(refsFromRaw(pull.getReferencesIn()), false),
      pInsert(refsFromRaw(pull.getReferencesOut()), true),
    ]);

    function pInsert(refs, outgoing) {
      return Promise.all(_.map(refs, function(ref) {
        return mysql.promiseQuery(
          'insert ignore into g_pull_references ' +
          (
            outgoing ?
            '  (from_pull_request_id, to_pull_request_id) values ' :
            '  (to_pull_request_id, from_pull_request_id) values '
          ) +
          '  (?, ' +
          '    pull_request_id_from_project_and_num( ' +
          '      project_id_from_owner_and_project_name( ' +
          '        owner_id_from_name(?), ? ' +
          '      ), ? ' +
          '    ) ' +
          '  ) ',
          [pull.id, ref.owner, ref.repo, ref.num]
        );
      }));
    }
  }

  function pInsertLabels(pull) {
    return Promise.all(_.map(pull.getLabels(), function(label) {
      return mysql.promiseQuery(
        'insert ignore into g_pull_labels (pull_request_id, label) values ' +
        '(?, ?)',
        [pull.id, label]
      );
    }));
  }

  function refsFromRaw(refs) {
    return _.map(refs, function(ref) {
      var sections = ref.split('/');
      assert(sections.length >= 4);
      var num = parseInt(sections[sections.length - 1]);
      assert(!isNaN(num));
      var owner = sections[sections.length - 4];
      assert(owner.length > 0);
      var repo = sections[sections.length - 3];
      assert(repo.length > 0);
      return {
        num: num,
        owner: owner,
        repo: repo,
      };
    });
  }

} else {
  process.env.WORKERS = 4;
  var lastPullId = -1;
  // process.env.MAX_PULL_ID = 10;
  process.env.PULLS_ACTIVE_PER_WORKER = 50;
  process.env.PULLS_COMPLETE_PER_WORKER = 1000;
  process.env.DEBUG = "*";

  var it = require(
      'lib/IterateOverPullFilesAndDoSomething/IterateOverPullFiles');

  it(__filename, lastPullId);
}
