/**
 * This round of parsing inserts:
 * - line diffs, (g_pull_descriptions#lines_(added|deleted))
 * - commit counts, (g_pull_descriptions#commit_count)
 * - file counts, (g_pull_descriptions#files_changed_count)
 * - merger_closer, (g_pull_descriptions#merger_closer)
 * - contributor statuses (g_pull_comments#author_status)
 */
if (module.parent) {
  module.exports = roundTwo;

  var mysql = require('lib/mysql'),
      _ = require('lodash'),
      util = require('util');

  function roundTwo(pull) {
    return Promise.all([
        pUpdateContributorStatus(pull.id, pull.getComments()),
        pUpdateDescription(pull),
    ]);
  }

  function pUpdateDescription(pull) {
    var params = [
      pull.getFilesChangedCount(),
      pull.getCommitCount(),
      pull.getLinesAddedCount(),
      pull.getLinesDeletedCount(),
      pull.getMergerCloser(),
      pull.id,
    ];

    if (isNaN(params[0]) || isNaN(params[1]) || isNaN(params[2]) ||
        isNaN(params[3])) {
      throw new Error(util.format('NaN value for pull %d', pull.id));
    }

    return mysql.promiseQuery(
      'update g_pull_descriptions ' +
      'set files_changed_count = ? ' +
      ',   commit_count = ? ' +
      ',   lines_added = ? ' +
      ',   lines_deleted = ? ' +
      ',   merger_closer = ? ' +
      'where pull_request_id = ? ',
      params
    ).then(function(res) {
      if (res.affectedRows !== 1) {
        throw new Error(util.format('could not pull request description ' +
            'id=%d'));
      }
    });
  }

  function pUpdateContributorStatus(id, comments) {
    return Promise.all(
      _.map(comments, function(comment, index) {
        mysql.promiseQuery(
          'update g_pull_comments ' +
          'set author_status = ? ' +
          'where author = ? ' +
          '  and `index` = ? ' +
          '  and pull_request_id = ? ',
          [comment.authorStatus, comment.author, index, id]
        ).then(function(res) {
          if (res.affectedRows !== 1) {
            throw new Error(util.format('could not find comment ' +
                'pid=%d,idx=%d,author=%s', id, index, comment.author));
          }
        });
      })
    );
  }
} else {
  process.env.WORKERS = 5;
  var lastPullId = 4039160;
  // process.env.MAX_PULL_ID = 10;
  process.env.PULLS_ACTIVE_PER_WORKER = 50;
  process.env.PULLS_COMPLETE_PER_WORKER = 1000;
  process.env.DEBUG = "*";

  var it = require(
      'lib/IterateOverPullFilesAndDoSomething/IterateOverPullFiles');

  it(__filename, lastPullId);
}
