require('binom')
library(scales)
library(ggplot2)
require(gridExtra)

profileAnalysis <- function(con){

	
query <-  sqlImport("perceived_gender_36/first.sql") 
	
di<-dbGetQuery(con, query)
do<-dbGetQuery(con, gsub("\\(gpc.creator_type in","!\\(gpc.creator_type in",query))

addCI <- function(d){
	conf <- binom.confint(d$merged, d$merged+d$closed+d$open, 0.95, 'exact');
	d$lower <- conf$lower
	d$upper <- conf$upper
	return(d)
}

di <- addCI(di)
do <- addCI(do)

plot.identifiable(di,do,'Publication - Conference - Gender Bias in Pull Requests/figs/gendered.pdf')

# install.packages('animation') # only run once
# Convert pdf to png
library("animation")
ani.options(outdir = getwd())
# im.convert("perceived_gender_36/second.pdf", output="perceived_gender_36/second.png", extra.opts="-density 150")

notes("Gendered vs. Gender-Neutral Acceptance?")

diffAndSig <- function(gendr){
	
	neutral <- subset(do,identifiable=='Gender-Neutral' & gender==gendr)
	gendered <- subset(do,identifiable=='Gendered' & gender==gendr)
	
	newcommandPercent(paste0(gendr,"OutsiderAcceptanceDifference"),abs(neutral$merge_rate-gendered$merge_rate))
	
	n <-  c(neutral$merged,neutral$closed+neutral$open)
	g <-  c(gendered$merged,gendered$closed+gendered$open)
	
	newcommand(paste0(gendr,"OutsiderAcceptanceSig"),chiSqFormat(chisq.test(rbind(n,g))))
}

diffAndSig("female")
diffAndSig("male")

makeDataTable <- function(shortened){
	shortened <- data.frame(identifiable=shortened$identifiable,
							gender = shortened$gender, 
							pulls = shortened$merged+shortened$closed+shortened$open,
							merge_rate=shortened$merge_rate,
							lower=shortened$lower,
							upper=shortened$upper)

	shortTab <- xtable(shortened,auto=TRUE)
					
	print(shortTab,
			hline.after=c(0),		
			include.rownames=FALSE,
			print.results=FALSE,
			comment=FALSE,
			floating=FALSE,
			tabular.environment = "longtable")
}

newcommand("genderedOutsiderTable",makeDataTable(do))
newcommand("genderedInsiderTable",makeDataTable(di))

return(list(di,do))
}

plot.identifiable <- function(di,do,filename){
	pli <- ggplot(data=di, aes(x=identifiable, ymin=lower, ymax=upper, fill=gender,
							lower=merge_rate, upper=merge_rate, middle=merge_rate, color=di$gender)) +
			geom_boxplot(stat='identity', position=position_dodge(0.05)) +
			scale_fill_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +  
			scale_colour_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +
			xlab("") +
			ylab("Acceptance Rate") +
			ggtitle("Insiders") +
			# geom_text(aes(label=merge_rate), vjust=1.6, position=position_dodge(0.9)) + 
			y_ar_scale() +
			theme_bw() +
			theme(legend.text = element_text(size = 12), legend.position = 'none', legend.direction = "horizontal", legend.key = element_rect(color = NA)) +
			theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
			theme(panel.border = element_rect(linetype = "blank")) +
			theme(aspect.ratio = 1) + 
			theme(panel.grid = element_line(linetype = "blank"),
					panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
					panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2),
					panel.grid.major.x = element_line(linetype = 'solid', color="#eeeeee", size=15)) +
			theme(plot.title=element_text(hjust=0.5))	
	
	plo <- ggplot(data=do, aes(x=identifiable, ymin=lower, ymax=upper, fill=gender,
							lower=merge_rate, upper=merge_rate, middle=merge_rate, color=do$gender)) +
			geom_boxplot(stat='identity', position=position_dodge(0.05)) +
			scale_fill_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +  
			scale_colour_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +
			y_ar_scale() +
			xlab("") +
			ggtitle("Outsiders") +
			# geom_text(aes(label=merge_rate), vjust=1.6, position=position_dodge(0.9)) + 
			theme_bw() +
			theme(legend.text = element_text(size = 12), legend.position = c(0,1.05), legend.direction = "horizontal", legend.background = element_rect(fill = NA), legend.key = element_rect(color = NA)) +
			theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
			theme(panel.border = element_rect(linetype = "blank")) +
			theme(aspect.ratio = 0.838) + #UGGGG! Should be way to make both fixed height 
			theme(panel.grid = element_line(linetype = "blank"),
					axis.text.y=element_blank(),
					panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
					panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2),
					panel.grid.major.x = element_line(linetype = 'solid', color="#eeeeee", size=15)) +
			theme(plot.title=element_text(hjust=0.5))
	
	pdf(filename,
			height=4.4)
	grid.arrange(pli, plo, ncol=2)
	dev.off()
	
	return(pli)
}