require(binom)
library(scales)
library(ggplot2)
require(gridExtra)

fileAnalysis <- function(con){
	
query <-  sqlImport("pr_file_types_issue_50/first.sql") 
d<-dbGetQuery(con, query)

d$merged = ifelse(d$state == 'merged', d$number_of_prs, 0)
d$closed = ifelse(d$state == 'closed', d$number_of_prs, 0)
d$open = ifelse(d$state == 'open', d$number_of_prs, 0)

d <- aggregate(d[,c('merged', 'closed', 'open')], by=list(d$gender, d$extension), FUN=sum)
d$gender = d$Group.1
d$extension = d$Group.2
d$total = d$merged + d$closed + d$open
d$merge_rate = d$merged / d$total
d$confidence = binom.confint(d$merged, d$total, 0.95, 'exact')
d = d[order(-d$total),]
# d$gender = ifelse(d$gender == 'Female', 'Women', 'Men')

# print(d[d$gender == 'Female',c('gender', 'extension', 'merged', 'closed', 'open', 'merge_rate')][1:50,])
# print(d[d$gender == 'Male',c('gender', 'extension', 'merged', 'closed', 'open', 'merge_rate')][1:50,])

tc_exts = c('.js', '.rb', '.py', '.php', '.java', '.cpp', '.cs', '.c', '.m', '.go')
ntc_exts = c('.md', '.html', '.json', '.xml', '.txt', '.rst', '.css', '.yml', '.markdown', '.podspec')

tc <- rbind(d[d$extension %in% tc_exts & d$gender == 'Female',][1:10,], d[d$extension %in% tc_exts & d$gender == 'Male',][1:10,])
ntc <- rbind(d[d$extension %in% ntc_exts & d$gender == 'Female',][1:10,], d[d$extension %in% ntc_exts & d$gender == 'Male',][1:10,])

addPs <- function(t){
	result <- c()
	for(ext in t$extension){
		short <- subset(t,extension==ext)
		result <- c(result,chisq.test(data.frame(accept=short$merged,reject=short$closed+short$open))$p.value)
	}
	t$p <- result
	return(t)
}

#### Add and adjust p values

tc 	<- addPs(tc)
ntc <- addPs(ntc)

ps <- rbind(	data.frame(unique(tc[,c("extension","p")]),type="tc"), #TODO don't need types, methinks
				data.frame(unique(ntc[,c("extension","p")]),type="ntc"))
ps$p.adj <- p.adjust(ps$p,"BH")

tc <- merge(tc,  ps,by="extension",all.x=TRUE)
ntc <- merge(ntc,ps,by="extension",all.x=TRUE)

tc$extension <- ifelse(tc$p.adj<0.05,paste(tc$extension,"*"),tc$extension)
ntc$extension <- ifelse(ntc$p.adj<0.05,paste(ntc$extension,"*"),ntc$extension)

####

tc$ordered_exts <- reorder(tc$extension, -tc$total)
ntc$ordered_exts <- reorder(ntc$extension, -ntc$total)


#print(tc[c('gender', 'extension', 'merged', 'closed', 'open', 'merge_rate')])
#print(ntc[c('gender', 'extension', 'merged', 'closed', 'open', 'merge_rate')])

ptc <- ggplot(data=tc, aes(x=ordered_exts, ymin=tc$confidence$lower, ymax=tc$confidence$upper, fill=gender,
			lower=merge_rate, upper=merge_rate, middle=merge_rate, color=tc$gender)) +
  geom_boxplot(stat='identity', position=position_dodge(0.05)) +
  scale_fill_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +  
  scale_colour_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +
  xlab("") +
  ylab("Acceptance Rate") +
  # geom_text(aes(label=merge_rate), vjust=1.6, position=position_dodge(0.9)) + 
  y_ar_scale() +
  theme_bw() +
  theme(legend.text = element_text(size = 12), legend.position = c(0.13,0.95), legend.direction = "horizontal", legend.key = element_rect(color = NA)) +
  theme(axis.text.x = element_text(angle = -38, hjust = 0)) +
  theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
  theme(panel.border = element_rect(linetype = "blank")) +
  theme(panel.grid = element_line(linetype = "blank"),
        panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
        panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2),
        panel.grid.major.x = element_line(linetype = 'solid', color="#eeeeee", size=15))

pntc <- ggplot(data=ntc, aes(x=ordered_exts, ymin=ntc$confidence$lower, ymax=ntc$confidence$upper, fill=gender,
			lower=merge_rate, upper=merge_rate, middle=merge_rate, color=ntc$gender)) +
  geom_boxplot(stat='identity', position=position_dodge(0.05)) +
  scale_fill_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +  
  scale_colour_manual(labels = c('Women', 'Men'), values=c('#ffbf80', '#660066'), name='') +
  xlab("") +
  ylab("Acceptance Rate") +
  # geom_text(aes(label=merge_rate), vjust=1.6, position=position_dodge(0.9)) + 
  y_ar_scale() +
  theme_bw() +
  theme(legend.position = 'none') +
  theme(axis.text.x = element_text(angle = -38, hjust = 0)) +
  theme(axis.line = element_line(size = 0.4, colour = "black", linetype = "solid")) +
  theme(panel.border = element_rect(linetype = "blank")) +
  theme(panel.grid = element_line(linetype = "blank"),
        panel.grid.major.y = element_line(linetype = 'dotted', color="#cccccc", size=0.3),
        panel.grid.minor.y = element_line(linetype = 'dotted', color="#cccccc", size=0.2),
        panel.grid.major.x = element_line(linetype = 'solid', color="#eeeeee", size=15))

pdf('Publication - Conference - Gender Bias in Pull Requests/figs/accepts_by_language.pdf')
grid.arrange(ptc, pntc, nrow=2)
dev.off()

# install.packages('animation') # only run once
# Convert pdf to png
library("animation")
ani.options(outdir = getwd())
# im.convert("pr_file_types_issue_50/third.pdf", output="pr_file_types_issue_50/third.png", extra.opts="-density 150")

shortened <- rbind(tc,ntc)
shortened <- data.frame(gender = shortened$gender, 
						pulls = shortened$merged+shortened$closed+shortened$open,
						extension=shortened$extension,
						merge_rate=shortened$merge_rate,
						lower=shortened$confidence$lower,
						upper=shortened$confidence$upper,
						p=shortened$p.x,
						p.adj=shortened$p.adj)
shortened <- shortened[order(shortened$extension),]

shortTab <- xtable(shortened,auto=TRUE)

newcommand(paste0("extTable"),
		print(shortTab,
				hline.after=c(0),		
				include.rownames=FALSE,
				print.results=FALSE,
				comment=FALSE,
				floating=FALSE,
				tabular.environment = "longtable"))
				
return(rbind(tc,ntc))
}