read_raw_dataset <- function(con) {
	dbGetQuery(con, sqlImport("psm_125/tsv_query.sql"))
}

load_dataset <- function(raw_d, pow = 2) {
  if (missing(raw_d)) raw_d <- read_raw_dataset()
  
  smooth <- function(x){
	  return(ifelse(x==0,-1,floor(log(x,pow))))
  }
  
  unique_female_extensions = unique(raw_d[raw_d$gender == 'Female',]$extension)
  raw_d = raw_d[raw_d$extension %in% unique_female_extensions,]
  
  #drop 'null' extensioned pulls (e.g. https://github.com/ntpl/ntpl/pull/1/files)
  raw_d = raw_d[raw_d$extension != 'null',]
  
  d <- data.frame(
    pull_request_id=as.integer(raw_d$pull_request_id),
    lines_added=as.integer(			smooth(raw_d$lines_added)),
    lines_deleted=as.integer(		smooth(raw_d$lines_deleted)),
    commit_count=as.integer(		smooth(raw_d$commit_count)),
    files_changed=as.integer(		smooth(raw_d$files_changed)),
    references_from=as.integer(		smooth(raw_d$references_from)),
	creator_pull_index=as.integer(	smooth(raw_d$creator_pull_index)),
	lines_added_raw=as.integer(		raw_d$lines_added),
	lines_deleted_raw=as.integer(	raw_d$lines_deleted),
	commit_count_raw=as.integer(	raw_d$commit_count),
	files_changed_raw=as.integer(	raw_d$files_changed),
	references_from_raw=as.integer(	raw_d$references_from),
	creator_pull_index_raw=as.integer(raw_d$creator_pull_index),
	license=factor(
		sapply(raw_d$license, function(x) {
					if (x == "Open Source") 1
					else if (x == "Other") 0
					else if (x == "No License") 0
					else stop(paste("invalid license type:", x))
				}),
		levels=c(0,1),
		labels=c("Not Open Source","Open Source")),
	extension=as.integer(factor(raw_d$extension)),
	extension_raw=raw_d$extension,
    identicon=factor(
        raw_d$identicon,
        levels=c(0,1),
        labels=c("custom", "identicon")),
	identifiable=raw_d$identifiable,
    creator_type=factor(
        sapply(raw_d$creator_type, function(x) {
          if (x == "Outsider") 0
          else if (x == "Collaborator") 1
          else if (x == "Owner") 2
          else stop(paste("invalid creator_type:", x))
        }),
        levels=c(0,1,2),
        labels=c("Outsider", "Collaborator", "Owner")),
    merge_state=factor(
        sapply(raw_d$state, function(x) {
          if (x == "open" || x == "closed") 0
          else if (x == "merged") 1
          else stop(paste("invalid merge state:", x))
        }),
        levels=c(0, 1),
        labels=c("Not merged", "Merged")),
    # matchit has a fit when
    # gender (match variable) is not integer 0 (control) or 1 (treatment)
    gender=as.integer(sapply(raw_d$gender, function(x) {
        if (x == "Male") 0
        else if (x == "Female") 1
        else stop(paste("invalid gender:", x))
      }))
  )
  # Just like in merge_state and gender, lets throw an error if the data-vector
  # (first argument to factor()) contains a value which was not in `levels`.
  if (any(is.na(d$identicon))) {
    stop("expected identicon to be in (0, 1)")
  }
  d
}