delimiter //
drop function if exists is_pull_merged;
create function is_pull_merged (pull_request_id int)
returns bool
begin

  declare project_id int;
  declare git_merged bool;
  declare gh_merged bool;
  declare head_commit_id int;

  -- get the project id
  select base_repo_id into project_id
  from pull_requests
  where id = pull_request_id;

  -- label this pull as merged if there is a github merged event for it
  select exists
  (
    select *
    from pull_request_history prh
    where prh.pull_request_id = pull_request_id
    and `action` = 'merged'
  ) into gh_merged;

  if(gh_merged) then
    return true;
  end if;

  select pull_request_head_commit(pull_request_id)
  into head_commit_id;

  -- consider the pull merged if the project the pull request was submitted to
  -- owns any commits where one of the parents is the pull_request's head
  -- commit
  select exists
  (
    select *
    from project_commits pc
    where pc.project_id = project_id
      and pc.commit_id in
      (
        -- select all commits where one of the parents is
        -- a commit from this pull request
        -- these aren't necessarily merges until the last step (above)
        select parents.commit_id
        from commit_parents parents
        where parents.parent_id = head_commit_id
      )
    limit 1
  ) into git_merged;

  return git_merged;
end//
delimiter ;
